/*
 *    config.c - Config system
 *
 *
 *    this file is part of:
 *
 *    JackDirector 0.1   ---   Start/Stop Transport by midi command, play a metronome
 *
 *    jack-director - 2014 (C) by Sergio Atzori <sergioatzori [at] gmail.com>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * system include files
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <unistd.h>

/*
 * my headers and definitions
 */

#include "jack-director.h"
#include "config.h"
#include "util.h"
#include "string.h"

void ReadConfigFile() {
	FILE *fhandle;
	short int eof;
	char *line,*cursor;
	
	if (access(config.config_file, F_OK) == -1) {
		printerror("Exit: Error: could not find configuration file: ", true, config.config_file);
		exit(1);  
	}
	printerror("Reading configuration file: ", false, config.config_file);

	if (fhandle = fopen(config.config_file,"r")) {
	} else {
		printerror("Exit: Error: could not open configuration file: ", true, config.config_file);
		exit(1);
	}

	eof=0;
	while (!eof) {
		line = GetLine(fhandle);
		if (*line == (char) EOF) {
			eof = 1;
// 			printerror("EOF", false, line);
		} else {
			cursor = Skip(line);
			if ((*cursor != COMMENT_SIGN) && (*cursor != '\0')) {
				ParseLine(cursor);
			}
// 			printerror("Line: ", false, line);
		}
		free(line);
	}

	if (fclose(fhandle)) {
		printerror("Exit: Error: could not close configuration file: ", true, config.config_file);
		exit(1);
	}
}

void SetDefaultValues() {
  config.debug = SHOW_DEBUG;
  config.config_file = AllocString(CONFIG_FILE);
  config.name = AllocString(SEQ_JD_NAME);
  config.mfreq = 900;
  config.mamplitude = 100;
  config.mduration = 20;
  config.mattack = 5;
  config.mdecay = 5;
  config.ntrans = 60;
  config.nstart = 62;
  config.nstop = 64;
  config.nthres = 0;
  config.midich = 0;
  config.bpm = 120;
  config.notimebase = 0;
  config.mcresyncdelay = 300;
  config.mcnotransport = 0;
  config.mcnoposition = 0;
  memset(midi_prg, 0, 16 * 128 * sizeof(MIDI_CMD));
}


void ParseLine(char *command) { /* Parses key=value1,value2,etc. */
  char *begin, *end, *begin_opt, *end_opt, *end_buffer, opt_msg[256], message[512], opts[32][128], *param;
  short int count = 0;
  
  Split(command,&begin,&end);
  param = begin;
  if (end == NULL) {
    printerror("Param without value: ", true, param);
    return;
  }
  sprintf(message,"command: %s = ", begin);
  end_buffer = end;
  end_opt = "a";
  while (end_opt != NULL) {
    SplitComma(end_buffer,&begin_opt,&end_opt);
    end_buffer = end_opt;
    if (begin_opt != NULL) {
      if (count) {
	sprintf(opt_msg, "%s - %s", opt_msg, begin_opt);
      } else {
	sprintf(opt_msg, "%s", begin_opt);
      }
      strcpy(opts[count], begin_opt);
      count++;
    }
  }
  if (count < 32) {
    *opts[count] = '\0';
  }
  sprintf(message,"%s%s", message, opt_msg);
  printerror(message, false, "");
  ParseConfig(param, opts);
  int nn;
}

void ParseConfig(char *param, char opts[32][128]) {
  char msgbuf[128];
  unsigned short ok = 0;
  int param0;
  char *option = opts[0];
  if (strcmp(param, "debug") == 0) {
    param0 = atoi(option);
    config.debug = (param0 > 4) ? 4 : param0;
    sprintf(msgbuf, "debug=%i", config.debug);
    ok = 1;
  } else if (strcmp(param, "mfreq") == 0) {
    param0 = atoi(option);
    config.mfreq = ((param0 > 20000) || (param0 < 20))? 900 : param0;
    sprintf(msgbuf, "mfreq=%i", config.mfreq);
    ok = 1;
  } else if (strcmp(param, "mamplitude") == 0) {
    param0 = atoi(option);
    config.mamplitude = ((param0 > 100) || (param0 < 1))? 100 : param0;
    sprintf(msgbuf, "mamplitude=%i", config.mamplitude);
    ok = 1;
  } else if (strcmp(param, "mduration") == 0) {
    param0 = atoi(option);
    config.mduration = ((param0 > 2000) || (param0 < 1))? 20 : param0;
    sprintf(msgbuf, "mduration=%i", config.mduration);
    ok = 1;
  } else if (strcmp(param, "mattack") == 0) {
    param0 = atoi(option);
    config.mattack = ((param0 > 100) || (param0 < 1))? 5 : param0;
    sprintf(msgbuf, "mattack=%i", config.mattack);
    ok = 1;
  } else if (strcmp(param, "mdecay") == 0) {
    param0 = atoi(option);
    config.mdecay = ((param0 > 100) || (param0 < 0))? 5 : param0;
    sprintf(msgbuf, "mdecay=%i", config.mdecay);
    ok = 1;
  } else if (strcmp(param, "ntrans") == 0) {
    param0 = atoi(option);
    config.ntrans = ((param0 > 127) || (param0 < 0))? 60 : param0;
    sprintf(msgbuf, "ntrans=%i", config.ntrans);
    ok = 1;
  } else if (strcmp(param, "nstart") == 0) {
    param0 = atoi(option);
    config.nstart = ((param0 > 127) || (param0 < 0))? 62 : param0;
    sprintf(msgbuf, "nstart=%i", config.nstart);
    ok = 1;
  } else if (strcmp(param, "nstop") == 0) {
    param0 = atoi(option);
    config.nstop = ((param0 > 127) || (param0 < 0))? 64 : param0;
    sprintf(msgbuf, "nstop=%i", config.nstop);
    ok = 1;
  } else if (strcmp(param, "nthres") == 0) {
    param0 = atoi(option);
    config.nthres = ((param0 > 127) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "nthres=%i", config.nthres);
    ok = 1;
  } else if (strcmp(param, "midich") == 0) {
    param0 = atoi(option);
    config.midich = ((param0 > 15) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "midich=%i", config.midich);
    ok = 1;
  } else if (strcmp(param, "bpm") == 0) {
    param0 = atoi(option);
    config.bpm = ((param0 > 400) || (param0 < 10))? 120 : param0;
    sprintf(msgbuf, "bpm=%i", config.bpm);
    ok = 1;
  } else if (strcmp(param, "notimebase") == 0) {
    param0 = atoi(option);
    config.notimebase = ((param0 > 1) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "notimebase=%i", config.notimebase);
    ok = 1;
  } else if (strcmp(param, "mcresyncdelay") == 0) {
    param0 = atoi(option);
    config.mcresyncdelay = ((param0 > 100000) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "mcresyncdelay=%i", config.mcresyncdelay);
    ok = 1;
  } else if (strcmp(param, "mcnotransport") == 0) {
    param0 = atoi(option);
    config.mcnotransport = ((param0 > 1) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "mcnotransport=%i", config.mcnotransport);
    ok = 1;
  } else if (strcmp(param, "mcnoposition") == 0) {
    param0 = atoi(option);
    config.mcnoposition = ((param0 > 1) || (param0 < 0))? 0 : param0;
    sprintf(msgbuf, "mcnoposition=%i", config.mcnoposition);
    ok = 1;
  } else if (strcmp(param, "name") == 0) {
    if (option != NULL) {
      config.name = AllocString(option);
      sprintf(msgbuf, "name=%s", config.name);
      ok = 1;
    }
  } else if (strstr(param, "prg")) {
    if (ok = ParseMidiCmd(param, opts)) {
      sprintf(msgbuf, "%s=OK", param);
    }
  }/* else if (strcmp(param, "") == 0) {
    
  }*/
  
  if (ok) {
    printerror("Success parsing: ", false, msgbuf);
  } else {
    printerror("Failed parsing: ", false, param);
  }
}

unsigned short ParseMidiCmd(char *param, char opts[32][128]) {
    char *attr, *val, msgbuf[128];
    short int ok = 0;
    short int cnt = 0;
    short int midich = config.midich;
    short int prgnum = 0;
    MIDI_CMD defmidicmd;
    defmidicmd.bpm = 120;
    defmidicmd.mute = 0;
    defmidicmd.stop = 0;
    defmidicmd.ignore = 0;
    defmidicmd.nolocate = 1;
    defmidicmd.locate = 0;
    defmidicmd.stopzero = 0;
    
    char *nprg = strndup(param + 3, strlen(param) - 3);
    if (strspn(nprg, "0123456789") == strlen(nprg)) {
      prgnum = atoi(nprg);
      if (prgnum <= 127) {
	sprintf(msgbuf, "%i -> ", prgnum);
	while (*opts[cnt] != NULL) {
	  SplitColon(opts[cnt], &attr,&val);
	  if (cnt > 0 && ok == 0) {
	    break;
	  }
	  if (strcmp(attr, "ignore") == 0) {
	    defmidicmd.ignore = ok = 1;
	  } else if (strcmp(attr, "stop") == 0) {
	    defmidicmd.stop = ok = 1;	    
	  } else if (strstr(attr, "mute")) {
	    defmidicmd.mute = ok = 1;
	  } else if (strstr(attr, "bpm") && val != NULL) {
	    if (strspn(val, "0123456789") == strlen(val)) {
	      int cval = atoi(val);
	      if (cval < 10 || cval > 400) {
		sprintf(msgbuf, "%s Invalid bpm value (10 - 400) ", msgbuf, val);
		ok = 0;
	      } else {
		defmidicmd.bpm = cval;
		ok = 1;
	      }
	    } else {
	      sprintf(msgbuf, "%s Invalid bpm value ", msgbuf, val);
	      ok = 0;
	    }
	  } else if (strstr(attr, "midich") && val != NULL) {
	    if (strspn(val, "0123456789") == strlen(val)) {
	      int cval = atoi(val);
	      if (cval < 0 || cval > 15) {
		sprintf(msgbuf, "%s Invalid midich value (0 - 15) ", msgbuf, val);
		ok = 0;
	      } else {
		midich = cval;
		ok = 1;
	      }
	    } else {
	      sprintf(msgbuf, "%s Invalid midich value ", msgbuf, val);
	      ok = 0;
	    }
	  } else if (strstr(attr, "locate") && val != NULL) {
	    if (strspn(val, "0123456789") == strlen(val)) {
	      int cval = atoi(val);
	      defmidicmd.locate = cval;
	      defmidicmd.nolocate = 0;
	      ok = 1;
	    } else {
	      sprintf(msgbuf, "%s Invalid locate value ", msgbuf, val);
	      ok = 0;
	    }
	  } else if (strcmp(attr, "stopzero") == 0) {
	    defmidicmd.stopzero = 1;
	  } else {
	    ok = 0;
	  }
	  if (ok) {
	    sprintf(msgbuf, "%s%s ", msgbuf, opts[cnt]);
	  } else {
	    sprintf(msgbuf, "%s Failed: %s", msgbuf, opts[cnt]);
	  }
	  cnt++;
	  free(attr);
	  free(val);
	}
      } else {
	sprintf(msgbuf, "Wrong program number (0 - 127): %i", prgnum);
      }
    } else {
      sprintf(msgbuf, "Wrong program number: %s", nprg);
    }
    printerror("--Parsing: ", false, msgbuf);
    if (ok) {
      midi_prg[midich][prgnum] = defmidicmd;
    }
    return ok;
}
